@@ -10,12 +10,24 @@ Below are some notes on how they are stored.
     [(assertion, compiled, return_val_var_name)].
 """
 
+from __future__ import annotations
+
 import inspect
 import logging
 import sys
 import typing
 from types import CodeType, FunctionType, ModuleType
-from typing import Any, Callable, List, Optional, Set, Tuple, TypeVar, Union, overload
+from typing import (
+    Any,
+    Callable,
+    Collection,
+    Optional,
+    TypeVar,
+    Union,
+    get_args,
+    get_origin,
+    overload,
+)
 
 import wrapt
 from typeguard import CollectionCheckStrategy, TypeCheckError, check_type
@@ -38,6 +50,11 @@ Set to False to disable workaround for PyCharm's "Run File in Python Console" ac
 In most cases you should not need to change this!
 """
 
+STRICT_NUMERIC_TYPES = True
+"""
+Set to False to allow more specific numeric types to be accepted by more general type annotations.
+"""
+
 _PYDEV_UMD_NAME = "pydev_umd"
 
 
@@ -103,16 +120,16 @@ Class = TypeVar("Class", bound=type)
 
 @overload
 def check_contracts(
-    func: FunctionType, module_names: Optional[Set[str]] = None
+    func: FunctionType, module_names: Optional[set[str]] = None
 ) -> FunctionType: ...
 
 
 @overload
-def check_contracts(func: Class, module_names: Optional[Set[str]] = None) -> Class: ...
+def check_contracts(func: Class, module_names: Optional[set[str]] = None) -> Class: ...
 
 
 def check_contracts(
-    func_or_class: Union[Class, FunctionType], module_names: Optional[Set[str]] = None
+    func_or_class: Union[Class, FunctionType], module_names: Optional[set[str]] = None
 ) -> Union[Class, FunctionType]:
     """A decorator to enable contract checking for a function or class.
 
@@ -183,12 +200,12 @@ def add_class_invariants(klass: type) -> None:
                 )
             except TypeCheckError:
                 raise AssertionError(
-                    f"Value {_display_value(value)} did not match type annotation for attribute "
-                    f"{name}: {_display_annotation(cls_annotations[name])}"
+                    f"Value {_display_value(value)} for attribute {name} did not match expected type "
+                    f"{_display_annotation(cls_annotations[name])}"
                 ) from None
         original_attr_value_exists = False
         original_attr_value = None
-        if hasattr(super(klass, self), name):
+        if hasattr(self, name):
             original_attr_value_exists = True
             original_attr_value = super(klass, self).__getattribute__(name)
         super(klass, self).__setattr__(name, value)
@@ -230,13 +247,16 @@ def _check_function_contracts(wrapped, instance, args, kwargs):
         if param in annotations:
             try:
                 _debug(f"Checking type of parameter {param} in call to {wrapped.__qualname__}")
-                check_type_strict(param, arg, annotations[param])
+                if STRICT_NUMERIC_TYPES:
+                    check_type_strict(param, arg, annotations[param])
+                else:
+                    check_type(arg, annotations[param])
             except (TypeError, TypeCheckError):
                 additional_suggestions = _get_argument_suggestions(arg, annotations[param])
 
                 raise PyTAContractError(
-                    f"{wrapped.__name__} argument {_display_value(arg)} did not match type "
-                    f"annotation for parameter {param}: {_display_annotation(annotations[param])}"
+                    f"Argument value {_display_value(arg)} for {wrapped.__name__} parameter {param} "
+                    f"did not match expected type {_display_annotation(annotations[param])}"
                     + (f"\n{additional_suggestions}" if additional_suggestions else "")
                 )
 
@@ -250,7 +270,7 @@ def _check_function_contracts(wrapped, instance, args, kwargs):
 
     # Check function preconditions
     if not hasattr(target, "__preconditions__"):
-        target.__preconditions__: List[Tuple[str, CodeType]] = []
+        target.__preconditions__: list[tuple[str, CodeType]] = []
         preconditions = parse_assertions(wrapped)
         for precondition in preconditions:
             try:
@@ -271,16 +291,19 @@ def _check_function_contracts(wrapped, instance, args, kwargs):
         return_type = annotations["return"]
         try:
             _debug(f"Checking return type from call to {wrapped.__qualname__}")
-            check_type_strict("return", r, return_type)
+            if STRICT_NUMERIC_TYPES:
+                check_type_strict("return", r, return_type)
+            else:
+                check_type(r, return_type)
         except (TypeError, TypeCheckError):
             raise PyTAContractError(
-                f"{wrapped.__name__}'s return value {_display_value(r)} did not match "
-                f"return type annotation {_display_annotation(return_type)}"
+                f"Return value {_display_value(r)} for {wrapped.__name__} did not match "
+                f"expected type {_display_annotation(return_type)}"
             )
 
     # Check function postconditions
     if not hasattr(target, "__postconditions__"):
-        target.__postconditions__: List[Tuple[str, CodeType, str]] = []
+        target.__postconditions__: list[tuple[str, CodeType, str]] = []
         return_val_var_name = _get_legal_return_val_var_name(
             {**wrapped.__globals__, **function_locals}
         )
@@ -308,17 +331,69 @@ def _check_function_contracts(wrapped, instance, args, kwargs):
 
 
 def check_type_strict(argname: str, value: Any, expected_type: type) -> None:
-    """Ensure that ``value`` matches ``expected_type``.
+    """Ensure that `value` matches ``expected_type`` with strict type checking.
 
-    Differentiates between:
-        - float vs. int
-        - bool vs. int
+    This function enforces strict type distinctions within the numeric hierarchy (bool, int, float,
+    complex), ensuring that the type of value is exactly the same as expected_type.
     """
-    if ENABLE_CONTRACT_CHECKING:
-        if (type(value) is int and expected_type is float) or (
-            type(value) is bool and expected_type is int
+    if not ENABLE_CONTRACT_CHECKING:
+        return
+    try:
+        _check_inner_type(argname, value, expected_type)
+    except (TypeError, TypeCheckError):
+        raise TypeError(f"type of {argname} must be {expected_type}; got {value} instead")
+
+
+def _check_inner_type(argname: str, value: Any, expected_type: type) -> None:
+    """Recursively checks if `value` matches `expected_type` for strict type validation, specifically supports checking
+    collections (list[int], dicts[float]) and Union types (bool | int).
+    """
+    inner_types = get_args(expected_type)
+    outer_type = get_origin(expected_type)
+    if outer_type is None:
+        if (
+            (type(value) is bool and expected_type in {int, float, complex})
+            or (type(value) is int and expected_type in {float, complex})
+            or (type(value) is float and expected_type is complex)
         ):
+            raise TypeError(
+                f"type of {argname} must be {expected_type}; got {type(value).__name__} instead"
+            )
+        else:
+            check_type(
+                value, expected_type, collection_check_strategy=CollectionCheckStrategy.ALL_ITEMS
+            )
+    elif outer_type is typing.Union:
+        for inner_type in inner_types:
+            try:
+                _check_inner_type(argname, value, inner_type)
+                return
+            except (TypeError, TypeCheckError):
+                pass
+        raise TypeError(f"type of {argname} must be {expected_type}; got {value} instead")
+    elif outer_type in {list, set}:
+        if isinstance(value, outer_type):
+            for item in value:
+                _check_inner_type(argname, item, inner_types[0])
+        else:
             raise TypeError(f"type of {argname} must be {expected_type}; got {value} instead")
+    elif outer_type is dict:
+        if isinstance(value, dict):
+            for key, item in value.items():
+                _check_inner_type(argname, key, inner_types[0])
+                _check_inner_type(argname, item, inner_types[1])
+        else:
+            raise TypeError(f"type of {argname} must be {expected_type}; got {value} instead")
+    elif outer_type is tuple:
+        if isinstance(value, tuple) and len(inner_types) == 2 and inner_types[1] is Ellipsis:
+            for item in value:
+                _check_inner_type(argname, item, inner_types[0])
+        elif isinstance(value, tuple) and len(value) == len(inner_types):
+            for item, inner_type in zip(value, inner_types):
+                _check_inner_type(argname, item, inner_type)
+        else:
+            raise TypeError(f"type of {argname} must be {expected_type}; got {value} instead")
+    else:
         check_type(
             value, expected_type, collection_check_strategy=CollectionCheckStrategy.ALL_ITEMS
         )
@@ -393,7 +468,7 @@ def _check_class_type_annotations(klass: type, instance: Any) -> None:
             )
         except TypeCheckError:
             raise AssertionError(
-                f"{_display_value(value)} did not match type annotation for attribute {attr}: "
+                f"Value {_display_value(value)} for attribute {attr} did not match expected type "
                 f"{_display_annotation(annotation)}"
             )
 
@@ -431,7 +506,7 @@ def _check_invariants(instance, klass: type, global_scope: dict) -> None:
                     curr_attributes = "{" + curr_attributes + "}"
 
                     raise PyTAContractError(
-                        f'"{instance.__class__.__name__}" representation invariant "{invariant}" was violated for'
+                        f'{instance.__class__.__name__} representation invariant "{invariant}" was violated for'
                         f" instance attributes {curr_attributes}"
                     )
 
@@ -504,14 +579,14 @@ def _check_assertions(
                 return_val_string = ""
 
                 if condition_type == "postcondition":
-                    return_val_string = f"and return value {function_return_val}"
+                    return_val_string = f" and return value {function_return_val}"
                 raise PyTAContractError(
                     f'{wrapped.__name__} {condition_type} "{assertion_str}" was '
-                    f"violated for arguments {arg_string} {return_val_string}"
+                    f"violated for arguments {arg_string}{return_val_string}"
                 )
 
 
-def parse_assertions(obj: Any, parse_token: str = "Precondition") -> List[str]:
+def parse_assertions(obj: Any, parse_token: str = "Precondition") -> list[str]:
     """Return a list of preconditions/postconditions/representation invariants parsed from the given entity's docstring.
 
     Uses parse_token to determine what to look for. parse_token defaults to Precondition.
@@ -638,7 +713,7 @@ def _debug(msg: str) -> None:
 def _set_invariants(klass: type) -> None:
     """Retrieve and set the representation invariants of this class"""
     # Update representation invariants from this class' docstring and those of its superclasses.
-    rep_invariants: List[Tuple[str, CodeType]] = []
+    rep_invariants: list[tuple[str, CodeType]] = []
 
     # Iterate over all inherited classes except builtins
     for cls in reversed(klass.__mro__):
