@@ -38,6 +38,9 @@ IGNORED_TESTS = [
     "e9950_forbidden_python_syntax.py",
 ]
 
+# The following tests require z3-solver as dependency
+Z3_RELATED_TESTS = ["r9900_redundant_condition.py", "r9901_impossible_condition.py"]
+
 
 def get_file_paths(paths: Union[str, list[str]]) -> list[str]:
     """
@@ -54,10 +57,21 @@ def get_file_paths(paths: Union[str, list[str]]) -> list[str]:
     if isinstance(paths, str):
         paths = [paths]
 
+    # check if z3 dependency is available
+    z3_dependency_available = True
+    try:
+        import z3
+    except ImportError:
+        z3_dependency_available = False
+
     for path in paths:
         for root, _, files in os.walk(path, topdown=True):
             for filename in files:
-                if filename not in IGNORED_TESTS and filename.endswith(".py"):
+                if (
+                    filename not in IGNORED_TESTS
+                    and filename.endswith(".py")
+                    and (z3_dependency_available or filename not in Z3_RELATED_TESTS)
+                ):
                     full_path = os.path.join(root, filename)
                     rel_path = os.path.relpath(full_path, path)
                     test_files.append(os.path.join(path, rel_path))
@@ -82,6 +96,7 @@ def _symbols_by_file_pyta(paths: list[str], include_msg: bool = False) -> dict[s
         config={
             "output-format": "python_ta.reporters.JSONReporter",
             "enable": ["C9960"],
+            "z3": True,
         },
     )
 
@@ -149,7 +164,7 @@ def test_examples_files_pyta(test_file: str, pyta_examples_symbols: dict[str, se
     found_pylint_message = checker_name in file_symbols
     assert (
         found_pylint_message
-    ), f"Failed {test_file}. File does not add expected message  {file_symbols}."
+    ), f"Failed {test_file}. File does not add expected message {checker_name}: {file_symbols}."
 
 
 @pytest.mark.parametrize("test_file", get_file_paths(_PYCODESTYLE_PATH))
